searchData={"items":[{"type":"module","title":"PairingHeap","doc":"Pairing Heap implementation\nsee:\n    http://en.wikipedia.org/wiki/Pairing_heap\n\nA Pairing Heap is a type of heap structure with relatively simple implementation and\nexcellent practical amortised performance.\n\nDetermining the precise asymptotic running time of pairing heaps has proved difficult,\nsee the Wikipedia page referenced above for a more complete discussion.\nIn particular practical performance of decrease-key is excellent (and initially\nconjectured to be O(1)), but at present it's known to be \"no worse\" then O(log n).\nHowever, no tight bound is known.\n\nOperation\n  find-min:     Θ(1)\n  delete-min:   Θ(log n)\n  insert:       Θ(1)\n  decrease-key: Θ(log n) - however, tight bound not known\n  merge:        Θ(1)\n\n\nGuts: pairing heaps\nA pairing heap is either nil or a term {key, value, [sub_heaps]}\nwhere sub_heaps is a list of heaps.\n\nTODO: Allow the comparison function to be specified\n      Implement decrease_key","ref":"PairingHeap.html"},{"type":"function","title":"PairingHeap.delete_min/1","doc":"return the heap with the min item removed\n\n    iex> PairingHeap.new(1, \"first\") |> PairingHeap.delete_min |> PairingHeap.empty?\n    true\n\n    iex> PairingHeap.new(1, \"first\") |> PairingHeap.delete_min |> PairingHeap.delete_min |> PairingHeap.empty?\n    true","ref":"PairingHeap.html#delete_min/1"},{"type":"type","title":"PairingHeap.element/0","doc":"","ref":"PairingHeap.html#t:element/0"},{"type":"function","title":"PairingHeap.empty?/1","doc":"True iff argument is an empty priority queue\n\n    iex> PairingHeap.new |> PairingHeap.empty?\n    true\n\n    iex> PairingHeap.new(1, \"first\") |> PairingHeap.empty?\n    false\n\n    iex> PairingHeap.new(1, \"first\") |> PairingHeap.delete_min |> PairingHeap.empty?\n    true","ref":"PairingHeap.html#empty?/1"},{"type":"type","title":"PairingHeap.key/0","doc":"","ref":"PairingHeap.html#t:key/0"},{"type":"function","title":"PairingHeap.meld/2","doc":"Merge (meld) two heaps","ref":"PairingHeap.html#meld/2"},{"type":"function","title":"PairingHeap.merge/2","doc":"Merge (meld) two heaps","ref":"PairingHeap.html#merge/2"},{"type":"function","title":"PairingHeap.min/2","doc":"min item in the heap","ref":"PairingHeap.html#min/2"},{"type":"function","title":"PairingHeap.new/0","doc":"Create new empty heap.\nOptionally pass in initial key, value","ref":"PairingHeap.html#new/0"},{"type":"function","title":"PairingHeap.new/2","doc":"","ref":"PairingHeap.html#new/2"},{"type":"function","title":"PairingHeap.pop/2","doc":"Returns the min item, as well as the heap without the min item\nEquivalent to:\n  {min(heap), delete_min(heap)}\n\n    iex> PairingHeap.new(1, \"first\") |> PairingHeap.pop |> elem(0)\n    {1, \"first\"}","ref":"PairingHeap.html#pop/2"},{"type":"function","title":"PairingHeap.put/3","doc":"Add element X to priority queue\n\n    iex> PairingHeap.new |> PairingHeap.put(1, \"first\") |> PairingHeap.pop |> elem(0)\n    {1, \"first\"}","ref":"PairingHeap.html#put/3"},{"type":"type","title":"PairingHeap.t/0","doc":"","ref":"PairingHeap.html#t:t/0"},{"type":"type","title":"PairingHeap.value/0","doc":"","ref":"PairingHeap.html#t:value/0"},{"type":"module","title":"PriorityQueue","doc":"Priority Queues in Elixir\n\nFrom Wikipedia: A priority queue is an abstract data type which is like\na regular queue or stack data structure, but where additionally each\nelement has a \"priority\" associated with it. In a priority queue, an\nelement with high priority is served before an element with low\npriority. If two elements have the same priority, they are served\naccording to their order in the queue.\n\nWhile priority queues are often implemented with heaps, they are\nconceptually distinct from heaps. A priority queue is an abstract\nconcept like \"a list\" or \"a map\"; just as a list can be implemented with\na linked list or an array, a priority queue can be implemented with a\nheap or a variety of other methods.\n\nIn computer science, a heap is a specialized tree-based data structure\nthat satisfies the heap property:\n*  If A is a parent node of B, then the key of node A is ordered with\n   respect to the key of node B, with the same ordering applying\n   across the heap.\n\na) Either the keys of parent nodes are always greater than or equal to\n   those of the children and the highest key is in the root node (this\n   kind of heap is called max heap)\nb) or the keys of parent nodes are less than or equal to those of the\n   children and the lowest key is in the root node (min heap)\n\nThe heap is one maximally efficient implementation of a Priority Queue\n\n%% heap :: nil | {Item, Value, [heap()]}","ref":"PriorityQueue.html"},{"type":"function","title":"PriorityQueue.delete_min/1","doc":"Return new priority queue with minimum element removed\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.delete_min |> PriorityQueue.size\n    3\n\n    iex> [{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.delete_min |> PriorityQueue.delete_min |> PriorityQueue.size\n    0","ref":"PriorityQueue.html#delete_min/1"},{"type":"function","title":"PriorityQueue.delete_min!/1","doc":"Return new priority queue with minimum element removed\n\nRaises a PriorityQueue.EmptyError exception if the queue is empty\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.delete_min |> PriorityQueue.size\n    3\n\n    iex> [{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.delete_min! |> PriorityQueue.delete_min! |> PriorityQueue.size\n    ** (PriorityQueue.EmptyError) queue empty error","ref":"PriorityQueue.html#delete_min!/1"},{"type":"type","title":"PriorityQueue.element/0","doc":"","ref":"PriorityQueue.html#t:element/0"},{"type":"function","title":"PriorityQueue.empty?/1","doc":"True iff argument is an empty priority queue\n\n    iex> PriorityQueue.new |> PriorityQueue.empty?\n    true\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.empty?\n    false","ref":"PriorityQueue.html#empty?/1"},{"type":"type","title":"PriorityQueue.key/0","doc":"","ref":"PriorityQueue.html#t:key/0"},{"type":"function","title":"PriorityQueue.keys/1","doc":"Retrieve keys from priority queue as a list in sorted order (may have duplicates)\n\nHeap sort a list\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.keys\n    [1, 3, 4, 8]","ref":"PriorityQueue.html#keys/1"},{"type":"function","title":"PriorityQueue.merge/2","doc":"Merge two priority queues\n\n    iex> PriorityQueue.merge( Enum.into([4,{8}], PriorityQueue.new), Enum.into([3,{1, \"first\"}], PriorityQueue.new)) |> PriorityQueue.to_list\n    [{1, \"first\"}, {3, nil}, {4, nil}, {8, nil}]","ref":"PriorityQueue.html#merge/2"},{"type":"function","title":"PriorityQueue.min/2","doc":"Return the minimum element\n\nIf the queue is empty, returns the default value ({nil, nil} if no default value)\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.min\n    {1, \"first\"}\n\n    iex> PriorityQueue.new |> PriorityQueue.min({:empty, nil})\n    {:empty, nil}","ref":"PriorityQueue.html#min/2"},{"type":"function","title":"PriorityQueue.min!/1","doc":"Return the minimum element\n\nRaises a PriorityQueue.EmptyError exception if the queue is empty\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.min!\n    {1, \"first\"}\n\n    iex> PriorityQueue.new |> PriorityQueue.min!\n    ** (PriorityQueue.EmptyError) queue empty error","ref":"PriorityQueue.html#min!/1"},{"type":"function","title":"PriorityQueue.new/0","doc":"Returns new, empty priority queue","ref":"PriorityQueue.html#new/0"},{"type":"function","title":"PriorityQueue.pop/2","doc":"Returns the min item, as well as the queue without the min item\n\nIf the queue is empty, returns the default value ({nil, nil} if no default value)\n\nEquivalent to:\n  {min(pq), delete_min(pq)}\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.pop |> elem(0)\n    {1, \"first\"}\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.pop |> elem(1) |> PriorityQueue.to_list\n    [{3, nil}, {4, nil}, {8, nil}]\n\n    iex> [{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.pop |> elem(1) |> PriorityQueue.pop({:empty, nil}) |> elem(0)\n    {:empty, nil}\n\n    iex> PriorityQueue.new |> PriorityQueue.pop({:empty, nil}) |> elem(0)\n    {:empty, nil}","ref":"PriorityQueue.html#pop/2"},{"type":"function","title":"PriorityQueue.pop!/1","doc":"Returns the min item, as well as the queue without the min item\n\nRaises a PriorityQueue.EmptyError exception if the queue is empty\n\nEquivalent to:\n  {min!(pq), delete_min!(pq)}\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.pop! |> elem(0)\n    {1, \"first\"}\n\n    iex> [{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.pop! |> elem(1) |> PriorityQueue.pop! |> elem(0)\n    ** (PriorityQueue.EmptyError) queue empty error\n\n    iex> PriorityQueue.new |> PriorityQueue.pop! |> elem(0)\n    ** (PriorityQueue.EmptyError) queue empty error","ref":"PriorityQueue.html#pop!/1"},{"type":"function","title":"PriorityQueue.put/2","doc":"Add (insert) element key,value to priority queue\nPass key/value either as two arguments, or as a tuple {key,value}\n\n    iex> PriorityQueue.new |> PriorityQueue.put(1) |> PriorityQueue.to_list\n    [{1, nil}]\n\n    iex> PriorityQueue.new |> PriorityQueue.put(1, \"first\") |> PriorityQueue.to_list\n    [{1, \"first\"}]\n\n    iex> PriorityQueue.new |> PriorityQueue.put({1, \"first\"}) |> PriorityQueue.to_list\n    [{1, \"first\"}]","ref":"PriorityQueue.html#put/2"},{"type":"function","title":"PriorityQueue.put/3","doc":"","ref":"PriorityQueue.html#put/3"},{"type":"function","title":"PriorityQueue.size/1","doc":"Number of elements in queue\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.size\n    4","ref":"PriorityQueue.html#size/1"},{"type":"type","title":"PriorityQueue.t/0","doc":"","ref":"PriorityQueue.html#t:t/0"},{"type":"function","title":"PriorityQueue.to_list/1","doc":"Retrieve elements from priority queue as a list in sorted order\n\n    iex> [4,{8},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.to_list\n    [{1, \"first\"}, {3, nil}, {4, nil}, {8, nil}]","ref":"PriorityQueue.html#to_list/1"},{"type":"type","title":"PriorityQueue.value/0","doc":"","ref":"PriorityQueue.html#t:value/0"},{"type":"function","title":"PriorityQueue.values/1","doc":"Retrieve values from priority queue as a list, sorted in order of their keys\n\n    iex> [4,{8, \"last\"},3,{1, \"first\"}] |> Enum.into(PriorityQueue.new) |> PriorityQueue.values\n    [\"first\", nil, nil, \"last\"]","ref":"PriorityQueue.html#values/1"},{"type":"exception","title":"PriorityQueue.EmptyError","doc":"","ref":"PriorityQueue.EmptyError.html"},{"type":"function","title":"PriorityQueue.EmptyError.message/1","doc":"","ref":"PriorityQueue.EmptyError.html#message/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}